@@ -36,7 +36,7 @@
 
 (defcustom telega-vvnote-waves-colors '("#006600" . "#229922")
   "Colors to display voice note waves."
-  :type 'cons
+  :type '(cons color color)
   :group 'telega-vvnote)
 
 (defcustom telega-vvnote-play-speed 1
@@ -51,13 +51,10 @@
   :type 'boolean
   :group 'telega-vvnote)
 
-(defun telega-vvnote-check-devices (how &rest devices)
-  "Check DEVICES is available in ffmpeg.
-Return list of available DEVICES."
-  (apply #'telega-ffplay--check-ffmpeg-output "-devices" how devices))
-
 (defconst telega-vvnote--has-audio-inputs
-  (telega-vvnote-check-devices '(encoder) "alsa" "pulse"))
+  (car (telega-ffplay--check-ffmpeg-output
+        "-devices" "alsa" "pulse"))
+  "List of audio inputs supported by telega.")
 
 (defcustom telega-vvnote-voice-record-args
   (concat (if (eq system-type 'darwin)
@@ -65,11 +62,11 @@ Return list of available DEVICES."
             ;; gnu/linux
             (concat "-f " (car telega-vvnote--has-audio-inputs)
                     " -i default"))
-          (cond ((member "opus1" telega-ffplay--has-encoders)
+          (cond ((telega-ffplay-has-encoder-p "opus")
                  ;; Try OPUS if available, results in 3 times smaller
                  ;; files then AAC version with same sound quality
                  " -strict -2 -acodec opus -ac 1 -ab 32k")
-                ((member "aac" telega-ffplay--has-encoders)
+                ((telega-ffplay-has-encoder-p "aac")
                  " -acodec aac -ac 1 -ab 96k")
                 (t
                  " -acodec mp3 -ar 44100 -ac 1 -ab 96k")))
@@ -85,11 +82,11 @@ Return list of available DEVICES."
             (concat "-f v4l2 -s 320x240 -i /dev/video0 -r 30 "
                     "-f " (car telega-vvnote--has-audio-inputs) " -i default"))
           " -vf format=yuv420p,scale=320:240,crop=240:240:40:0"
-          " -vcodec " (if (member "hevc" telega-ffplay--has-encoders)
+          " -vcodec " (if (telega-ffplay-has-encoder-p "hevc")
                           "hevc"
                         "h264")
           " -vb 300k"
-          (if (member "opus" telega-ffplay--has-encoders)
+          (if (telega-ffplay-has-encoder-p "opus")
               " -strict -2 -acodec opus -ac 1 -ab 32k"
             ;; Fallback to aac
             " -acodec aac -ac 1 -ab 96k"))
@@ -111,16 +108,16 @@ Return list of available DEVICES."
          (n-factor (/ (- max-o min-o) (apply 'max min-normw))))
     (mapcar (lambda (v) (+ min-o (* v n-factor))) min-normw)))
 
-(defun telega-vvnote--waves-svg (waves height duration &optional played)
-  "From decoded WAVES create svg of HEIGHT for DURATION and PLAYED."
-  (cl-assert (> height 8))
-  (let* ((w-idx 0)
+(defun telega-vvnote--waves-svg (waves duration &optional played)
+  "From decoded WAVES create svg for DURATION and PLAYED."
+  (let* ((w1 20)
+         (height 30)
+         (w-idx 0)
          (wv-width 3) (space-width 2)
          (wv-height (- height 6))
          (w (* (+ wv-width space-width) (length waves)))
-         (aw-chars (telega-chars-in-width w))
-         (cw (telega-chars-xwidth aw-chars))
-         (svg (telega-svg-create cw height)))
+         (aw-chars (max 5 (ceiling (/ w w1))))
+         (svg (telega-svg-create w height)))
     ;; bg - "#e1ffc7", fg - "#93d987", fg-played - "#3fc33b"
     ;;    (svg-rectangle svg 0 0 w h :fill-color "#e1ffc7")
     (dolist (wv waves)
@@ -135,12 +132,17 @@ Return list of available DEVICES."
                   :stroke-width (if played-p (1+ wv-width) wv-width)
                   :stroke-linecap "round")
         (cl-incf w-idx)))
-    (telega-svg-image svg :scale 1
-                      :width cw :height height
-                      :mask 'heuristic
-                      :ascent 'center
-                      ;; text of correct width
-                      :telega-text (make-string aw-chars ?#))))
+    (telega-svg-image svg
+      :scale 1
+      :height (telega-ch-height telega-vvnote-waves-height-factor)
+      :width (telega-cw-width aw-chars)
+      :mask 'heuristic
+      :ascent 'center
+      :telega-text
+      (telega-ins--as-string
+       (telega-ins "[")
+       (telega-ins-progress-bar played duration (- aw-chars 2) ?\# ?\.)
+       (telega-ins "]")))))
 
 ;; Encoding and Decoding splits into two situations:
 ;;     head-bits=5   tail-bits=0
@@ -167,10 +169,10 @@ Unless RAW-P is given, heights are normalized to [0-1] values."
              (bwv1 (if (> (length bwv) 1) (aref bwv 1) 0))
              (head-bits (min 5 (- 8 bitoff)))
              (tail-bits (- 5 head-bits))
-             (h-value (lsh (logand bwv0 (lsh 255 (- bitoff)))
+             (h-value (ash (logand bwv0 (ash 255 (- bitoff)))
                            (- (+ bitoff head-bits) 8)))
-             (t-value (lsh bwv1 (- tail-bits 8))))
-        (setq result (cons (logior (lsh h-value tail-bits) t-value) result)
+             (t-value (ash bwv1 (- tail-bits 8))))
+        (setq result (cons (logior (ash h-value tail-bits) t-value) result)
               bitoff (+ bitoff 5))
         (when (>= bitoff 8)
           (cl-assert (= (- bitoff 8) tail-bits))
@@ -189,17 +191,17 @@ WAVEFORM is list of integers each in range [0-31] to fit into 5 bits."
       (cl-assert (< bitoff 8))
       (let* ((head-bits (min 5 (- 8 bitoff)))
              (tail-bits (- 5 head-bits))
-             (h-value (lsh (logand w-value (lsh 31 tail-bits)) (- tail-bits)))
-             (t-value (logand w-value (lsh 31 (- head-bits)))))
+             (h-value (ash (logand w-value (ash 31 tail-bits)) (- tail-bits)))
+             (t-value (logand w-value (ash 31 (- head-bits)))))
         ;; Write head-bits
-        (setq value (logior value (lsh h-value (- 8 (+ bitoff head-bits))))
+        (setq value (logior value (ash h-value (- 8 (+ bitoff head-bits))))
               bitoff (+ bitoff 5))
         ;; Write tail bits
         (when (>= bitoff 8)
           (cl-assert (= (- bitoff 8) tail-bits))
           (setq bytes (cons value bytes)
                 bitoff (- bitoff 8)
-                value (lsh t-value (- 8 bitoff))))))
+                value (ash t-value (- 8 bitoff))))))
 
     (base64-encode-string (apply #'unibyte-string (nreverse bytes)) t)))
 
@@ -286,20 +288,24 @@ Each sample is in range [-128;128]."
     (cl-assert (cl-every (apply-partially #'>= 31) n-waves))
     (telega-vvnote--waveform-encode n-waves)))
 
-(defun telega-vvnote-video--svg (framefile &optional progress
-                                           data-p frame-img-type)
+(cl-defun telega-vvnote-video--svg (framefile &key as-data-image-type
+                                              progress with-noise-p)
   "Generate svg image for the video note FRAMEFILE.
 PROGRESS is value from 0 to 1 indicating played content.
 PROGRESS might be nil, meaning video not is not started.
 PROGRESS also can be a cons cell, where car is `paused' and cdr is
 value from 0 to 1, meaning video note is paused at given progress.
-If DATA-P is non-nil then FRAMEFILE is actually an image data.
-If DATA-P is non-nil then FRAME-IMG-TYPE specifies type of the image."
-  (let* ((img-type (or frame-img-type (image-type-from-file-name framefile)))
-         (size (telega-chars-xheight
-                (if (consp telega-video-note-height)
-                    (car telega-video-note-height)
-                  telega-video-note-height)))
+If AS-DATA-IMAGE-TYPE is non-nil, then FRAMEFILE is actually an image
+data and AS-DATA-IMAGE-TYPE specifies type of the image.
+If WITH-NOISE-P is non-nil, then use noise filter above the image."
+  (declare (indent 1))
+  (let* ((data-p (when as-data-image-type t))
+         (img-type (or as-data-image-type
+                       (telega-image-supported-file-p framefile)))
+         (ah-chars (if (consp telega-video-note-height)
+                       (car telega-video-note-height)
+                     telega-video-note-height))
+         (size (telega-chars-xheight ah-chars))
          (h size)
          (aw-chars (telega-chars-in-width size))
          (w (telega-chars-xwidth aw-chars))
@@ -310,6 +316,8 @@ If DATA-P is non-nil then FRAME-IMG-TYPE specifies type of the image."
          (clip1 (telega-svg-clip-path svg "clip1"))
          (playing-p (numberp progress)))
     (svg-circle clip (/ w 2) (/ h 2) (/ size 2))
+    (when with-noise-p
+      (telega-svg-append-spoiler-node svg "noise"))
     (telega-svg-embed svg (if data-p
                               framefile
                             (list (file-name-nondirectory framefile)
@@ -317,6 +325,8 @@ If DATA-P is non-nil then FRAME-IMG-TYPE specifies type of the image."
                       (format "image/%S" img-type) data-p
                       :x xoff :y yoff
                       :width size :height size
+                      :filter (when with-noise-p
+                                "url(#noise)")
                       :clip-path "url(#clip)")
 
     (when progress
@@ -352,35 +362,41 @@ If DATA-P is non-nil then FRAME-IMG-TYPE specifies type of the image."
 
     ;; Draw play triangle
     (unless playing-p
-      (let ((play-size (/ size 6)))
-        (svg-polygon svg (list (cons (/ (- size play-size) 2)
-                                     (/ (- size play-size) 2))
-                               (cons (/ (- size play-size) 2)
-                                     (/ (+ size play-size) 2))
-                               (cons (/ (+ size play-size) 2)
-                                     (/ size 2)))
+      (let ((play-size (/ h 6)))
+        (svg-polygon svg (list (cons (/ (- w play-size) 2)
+                                     (/ (- h play-size) 2))
+                               (cons (/ (- w play-size) 2)
+                                     (/ (+ h play-size) 2))
+                               (cons (/ (+ w play-size) 2)
+                                     (/ h 2)))
                      :fill "red"
                      :opacity "0.5")))
 
-    (telega-svg-image svg :scale 1.0
-               :base-uri (if data-p "" framefile)
-               :width w :height h
-               :mask 'heuristic
-               :ascent 'center
-               :telega-text (make-string aw-chars ?#))))
+    (telega-svg-image svg
+      :scale 1.0
+      :height (telega-ch-height ah-chars)
+      ;; NOTE: Do not use `:telega-nslices' because when watching
+      ;; video note it grows in size
+;      :telega-nslices ah-chars
+      :mask 'heuristic
+      :ascent 'center
+      :base-uri (if data-p "" framefile)
+      :telega-text (make-string aw-chars ?#))))
 
-(defun telega-vvnote-video--create-image (note &optional _file)
+(defun telega-vvnote-video--create-image (note &optional _file with-noise-p)
   "Create image for video NOTE frame."
   (let* ((thumb (plist-get note :thumbnail))
          (thumb-file (telega-file--renew thumb :file))
          (minithumb (plist-get note :minithumbnail)))
     (cond ((telega-file--downloaded-p thumb-file)
            (telega-vvnote-video--svg
-            (telega--tl-get thumb-file :local :path)))
+               (telega--tl-get thumb-file :local :path)
+             :with-noise-p with-noise-p))
           (minithumb
            (telega-vvnote-video--svg
-            (base64-decode-string (plist-get minithumb :data))
-            nil t 'jpeg))
+               (base64-decode-string (plist-get minithumb :data))
+             :as-data-image-type 'jpeg
+             :with-noise-p with-noise-p))
           (t
            (let* ((cheight (if (consp telega-video-note-height)
                                (car telega-video-note-height)
@@ -388,6 +404,17 @@ If DATA-P is non-nil then FRAME-IMG-TYPE specifies type of the image."
                   (x-size (telega-chars-xheight cheight)))
              (telega-media--progress-svg thumb-file x-size x-size cheight))))))
 
+(defun telega-vvnote-video-ttl--create-image (note &optional file)
+  (telega-vvnote-video--create-image note file :with-noise))
+
+(defun telega-vvnote-video--create-image-one-line (note &optional file)
+  (let ((telega-video-note-height 0.8))
+    (telega-vvnote-video--create-image note file)))
+
+(defun telega-vvnote-video-ttl--create-image-one-line (note &optional file)
+  (let ((telega-video-note-height 0.8))
+    (telega-vvnote-video--create-image note file :with-noise)))
+
 
 ;; Recording notes
 (defvar telega-vvnote--record-progress nil
@@ -463,21 +490,21 @@ Return filename with recorded voice note."
         ;; Copy first frame
         (when (= 1 (car frame))
           (let ((first-frame (telega-temp-name "video-note1" ".png")))
-            (copy-file (cdr frame) first-frame)
+            (copy-file (cdr frame) first-frame 'ok-if-exists)
             (plist-put telega-vvnote-video--preview :first-frame first-frame)))
 
         ;; Display frame
         (telega-ins--image
-         (telega-vvnote-video--svg (cdr frame) nil nil 'png))))))
+         (telega-vvnote-video--svg (cdr frame)))))))
 
 (defun telega-vvnote-video--record ()
   "Record a video note.
 Return filename with recorded video note."
   ;; Check codecs availability
-  (when (or (and (not (member "opus" telega-ffplay--has-encoders))
-                 (not (member "aac" telega-ffplay--has-encoders)))
-            (and (not (member "hevc" telega-ffplay--has-encoders))
-                 (not (member "h264" telega-ffplay--has-encoders))))
+  (when (or (and (not (telega-ffplay-has-encoder-p "opus"))
+                 (not (telega-ffplay-has-encoder-p "aac")))
+            (and (not (telega-ffplay-has-encoder-p "hevc"))
+                 (not (telega-ffplay-has-encoder-p "h264"))))
     (user-error "telega: sorry, your ffmpeg can't record video notes"))
 
   (setq telega-vvnote-video--preview
@@ -579,28 +606,25 @@ I.e. if you press 7, then you will jump to 70% of the message
 duration."
   (interactive (list (telega-msg-for-interactive)))
 
-  (when-let ((proc (plist-get msg :telega-ffplay-proc)))
-    (let* ((content (plist-get msg :content))
-           (duration (cl-case (telega--tl-type content)
-                       (messageVoiceNote
-                        (telega--tl-get content :voice_note :duration))
-                       (messageVideoNote
-                        (telega--tl-get content :video_note :duration))
-                       (messageAudio
-                        (telega--tl-get content :audio :duration))
-                       (t
-                        ;; Possibly a element embedded into webpage
-                        (let ((web-page (plist-get content :web_page)))
-                          (plist-get (or (plist-get web-page :audio)
-                                         (plist-get web-page :video_note)
-                                         (plist-get web-page :voice_note))
-                                     :duration)))))
-           (n-part (string-to-number (this-command-keys))))
-      (cl-assert (< 0 n-part 10))
-      (when (and (telega-ffplay-playing-p proc) duration)
-        (telega-ffplay-pause
-         proc (floor (* (/ n-part 10.0) duration)) 'ignore-callback)
-        (telega-msg-open-content msg)))))
+  (when-let ((proc (plist-get msg :telega-ffplay-proc))
+             (duration
+              (plist-get
+               (or (plist-get (telega-msg-match-p msg
+                                '(or (type VoiceNote) (link-preview VoiceNote)))
+                              :voice_note)
+                   (plist-get (telega-msg-match-p msg
+                                '(or (type VideoNote) (link-preview VideoNote)))
+                              :video_note)
+                   (plist-get (telega-msg-match-p msg
+                                '(or (type Audio) (link-preview Audio)))
+                              :audio))
+               :duration))
+             (n-part (string-to-number (this-command-keys))))
+    (cl-assert (< 0 n-part 10))
+    (when (and (telega-ffplay-playing-p proc) duration)
+      (telega-ffplay-pause
+       proc (floor (* (/ n-part 10.0) duration)) 'ignore-callback)
+      (telega-msg-open-content msg))))
 
 (provide 'telega-vvnote)
 
