@@ -8,12 +8,12 @@
 ;; Keywords: comm
 ;; Package-Requires: ((emacs "27.1") (visual-fill-column "1.9") (transient "0.3.0"))
 ;; URL: https://github.com/zevlg/telega.el
-;; Package-Version: 20251023.1131
-;; Package-Revision: c8000bb5a954
-(defconst telega-version "0.8.554")
+;; Package-Version: 20251122.2249
+;; Package-Revision: e88f8b98f7ca
+(defconst telega-version "0.8.560")
 (defconst telega-server-min-version "0.7.7")
-(defconst telega-tdlib-min-version "1.8.55")
-(defconst telega-tdlib-max-version "1.8.55")
+(defconst telega-tdlib-min-version "1.8.56")
+(defconst telega-tdlib-max-version nil)
 
 (defconst telega-tdlib-releases '("1.8.0" . "1.9.0")
   "Cons cell with current and next TDLib releases.
@@ -395,7 +395,6 @@ string at point."
 (telega-active-video-chats-mode 1)
 (telega-contact-birthdays-mode 1)
 (telega-active-stories-mode 1)
-(telega-network-stats-mode 1)
 
 ;; Enable root auto fill mode by default
 (telega-root-auto-fill-mode 1)
